
// sp_spi.c :  Plug-in for playing prerecorded ogg files from Intel, Eon,
// Winbond or equivalent 128Kbyte...16Mbyte SPI serial flash eeprom. 
// For this example, a serial flash chip is connected to SI, SO, SCLK, XCS.
// LEDS are connected to GPIO0[12] (NFCLE) and GPIO0[13] (NFALE) pins

// FAT image is expected to start at sector 32 (16K after chip start)

#include <stdlib.h>
#include <vs1000.h>

const u_int16 clockDivider = 2;
const u_int16 fatStartSector = 32;

#define SPI_EEPROM_COMMAND_READ_STATUS_REGISTER  0x05
#define SPI_EEPROM_COMMAND_READ  0x03

//macro to set SPI to MASTER; 8BIT; FSYNC Idle => xCS high
#define SPI_MASTER_8BIT_CSHI   PERIP(SPI0_CONFIG) = \
        SPI_CF_MASTER | SPI_CF_DLEN8 | SPI_CF_FSIDLE1

//macro to set SPI to MASTER; 8BIT; FSYNC not Idle => xCS low
#define SPI_MASTER_8BIT_CSLO   PERIP(SPI0_CONFIG) = \
        SPI_CF_MASTER | SPI_CF_DLEN8 | SPI_CF_FSIDLE0

//macro to set SPI to MASTER; 16BIT; FSYNC not Idle => xCS low
#define SPI_MASTER_16BIT_CSLO  PERIP(SPI0_CONFIG) = \
        SPI_CF_MASTER | SPI_CF_DLEN16 | SPI_CF_FSIDLE0


void InitSpi() {
  SPI_MASTER_8BIT_CSHI;
  PERIP(SPI0_FSYNC) = 0;        // Frame Sync is used as an active low xCS
  PERIP(SPI0_CLKCONFIG) = SPI_CC_CLKDIV * (clockDivider-1);   // Spi clock divider = 1
  PERIP(GPIO1_MODE) |= 0x1f;    // Set SPI pins to be peripheral controlled
}  


void EESingleCycleCommand(u_int16 cmd){
  SPI_MASTER_8BIT_CSHI; 
  SPI_MASTER_8BIT_CSLO;
  SpiSendReceive(cmd);
  SPI_MASTER_8BIT_CSHI;
}


/// Wait for not_busy (status[0] = 0) and return status
u_int16 EEWaitGetStatus(void) {
  u_int16 status;
  SPI_MASTER_8BIT_CSHI;
  SPI_MASTER_8BIT_CSLO;
  SpiSendReceive(SPI_EEPROM_COMMAND_READ_STATUS_REGISTER);
  while ((status = SpiSendReceive(0)) & 0x01)
    ; //Wait until ready
  SPI_MASTER_8BIT_CSHI;
  return status;
}


/// Read a block from EEPROM
/// \param blockn number of 512-byte sector 0..32767
/// \param dptr pointer to data block
u_int16 EEReadBlock(u_int16 blockn, u_int16 *dptr) {
  EEWaitGetStatus();                    // Wait until EEPROM is not busy
  SPI_MASTER_8BIT_CSLO;                  // Bring xCS low
  SpiSendReceive(SPI_EEPROM_COMMAND_READ);
  SpiSendReceive(blockn>>7);            // Address[23:16] = blockn[14:7]
  SpiSendReceive((blockn<<1)&0xff);     // Address[15:8]  = blockn[6:0]0
  SpiSendReceive(0);                    // Address[7:0]   = 00000000
  SPI_MASTER_16BIT_CSLO;                 // Switch to 16-bit mode
  { int n;
    for (n=0; n<256; n++){
      *dptr++ = SpiSendReceive(0);      // Receive Data
    }
  }
  SPI_MASTER_8BIT_CSHI;                  // Bring xCS back to high
  return 0;
}

// This function will replace ReadDiskSector() functionality
auto u_int16 MyReadDiskSector(register __i0 u_int16 *buffer,
			      register __a u_int32 sector) {

  PERIP(GPIO0_DDR) |= 1<<12; // LED1 pin is output
  PERIP(GPIO0_ODATA) |= 1<<12; // LED1 is high
  PERIP(GPIO1_MODE) |= 0x1f;    // Set SPI pins to be peripheral controlled
  EEReadBlock(sector+fatStartSector, buffer);
  PERIP(GPIO0_DDR) &= ~(1<<12); // LED1 pin is input
  PERIP(GPIO0_ODATA) &= ~(1<<12); // LED1 is low (in case someone makes it output)

  return 0;
}


// Initialize SPI and hook in own disk read function.
void main(void) {
  InitSpi();

  // Hook in own disk sector read function
  SetHookFunction((u_int16)ReadDiskSector, MyReadDiskSector);

} // Return to ROM code. Player will now play from EEPROM

// Note that you are encouraged to insert your own functionality
// as needed instead of dropping to the ROM code. You can also
// replace the main player loop (copy relevant code from the
// playloop.c example in vskit133 or later package) to control
// playing directly (instead of the default user interface).
